from .about import AboutCommand
from .add import AddCommand
from .install import InstallCommand
from .lock import LockCommand
from .new import NewCommand
from .remove import RemoveCommand
from .update import UpdateCommand
